"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _apolloDatasourceRest = require("apollo-datasource-rest");

var _lodash = require("lodash");

var _generatePassword = require("generate-password");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class ServerClient extends _apolloDatasourceRest.RESTDataSource {
  constructor(props) {
    super();

    _defineProperty(this, "managementToken", async () => {
      try {
        if (!this.tokenExpired) return this._managementToken;
        const data = await this.post('/oauth/token', {
          client_id: this.props.clientId,
          client_secret: this.props.clientSecret,
          audience: `https://${this.props.domain}/api/v2/`,
          grant_type: 'client_credentials'
        }, {
          cacheOptions: {
            ttl: 0
          }
        });
        this._managementToken = data.access_token;
        this._tokenExpiresAt = Date.now() + data.expires_in * 1000;
        return this._managementToken;
      } catch (err) {
        throw (0, _lodash.get)(err, 'response.data.message') || (0, _lodash.get)(err, 'message') || err;
      }
    });

    this.props = _objectSpread({
      ttl: 60,

      willSendRequest() {}

    }, props);
    if (!this.props.domain) throw new Error('Auth0 ServerClient domain is required');
    if (!this.props.clientId) throw new Error('Auth0 ServerClient management clientId is required');
    if (!this.props.clientSecret) throw new Error('Auth0 ServerClient management clientSecret is required');
    this.baseURL = `https://${this.props.domain}`;
  }

  async willSendRequest(request) {
    try {
      // Prevent an infinite loop (caused by the management token request)
      if (request.path === '/oauth/token') return;
      const managementToken = await this.managementToken();
      request.headers.set('Authorization', `Bearer ${managementToken}`);
      this.props.willSendRequest(request);
    } catch (err) {
      throw err;
    }
  }

  get tokenExpired() {
    if (!this._tokenExpiresAt) return true;
    return Date.now() > this._tokenExpiresAt;
  }

  getUsers(params) {
    return this.get('/api/v2/users', _objectSpread({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  getUser(id, params) {
    return this.get(`/api/v2/users/${id}`, _objectSpread({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  getUsersByEmail(params) {
    return this.get('/api/v2/users-by-email', _objectSpread({}, params, {
      search_engine: 'v3'
    }), {
      cacheOptions: {
        ttl: this.props.ttl
      }
    });
  }

  async createUser(user) {
    try {
      const res = await this.post('/api/v2/users', _objectSpread({
        connection: 'Username-Password-Authentication',
        password: (0, _generatePassword.generate)({
          length: 10,
          numbers: true,
          uppercase: true
        })
      }, user));
      return res;
    } catch (err) {
      throw (0, _lodash.get)(err, 'extensions.response.body.message') || err;
    }
  }

  async resetPassword(id, params) {
    try {
      const {
        newPassword
      } = params;
      const res = await this.patch(`/api/v2/users/${id}`, {
        password: newPassword,
        connection: 'Username-Password-Authentication'
      }, {
        cacheOptions: {
          ttl: 0
        }
      });
      return res;
    } catch (err) {
      throw (0, _lodash.get)(err, 'extensions.response.body.message') || err;
    }
  }

  sendChangePasswordEmail(email) {
    return this.post('/dbconnections/change_password', {
      email,
      connection: 'Username-Password-Authentication'
    }, {
      cacheOptions: {
        ttl: 0
      }
    });
  }

}

exports.default = ServerClient;